'Filename   Taster2_104.bas
'Version    1.04
'Date       2013-12-06
'Purpose    Test of "Tasten entprellen - Bulletproof"
'           http://www.mikrocontroller.net/topic/tasten-entprellen-bulletproof
'           Timer0 interrupt is used
'           Test Key_port at 2 different AVR ports
'           One key (here key0) is checked for pressing short/long
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega8
'           Crystal 16 MHz
'           2x16 Text LCD @ PortC & D
'           4 keys @ PortC
'Compiler   BASCOM-AVR 2.0.7.3
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           Flash used: 1524 bytes (18%)
'Edit history
'V1.02      2013-10-25, first release, upgrade of simple V1.00
'V1.03      2013-11-24, keys at different ports
'V1.04      2013-12-06, improved long/short key pressing

'==============================================================================

'Fuse bits  can be set with AVR Studio 4.0 & ISP-PROG via USB
'           FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'           In ATMEGA-Data sheets:  Bit = 1 -> not set
'                                   Bit = 0 -> set
'           RSTDISBL   [ ]
'           WTDON      [ ]
'           SPIEN      [x]
'           EESAVE     [ ]
'           BOOTSZ     Boot Flash size=128 words (minimal boot size)
'           BOOTRST    [ ]
'           CKOPT      [ ]
'           BODLEVEL   Brown-out detection at VCC=2.7V
'           BODDEN     [ ]
'           SUT_CKSEL  Ext. Crystal Osc. High Freq., Start-up time 16k + 64ms

$regfile = "m8def.dat"                            'ATmega8
$crystal = 16000000                               '16 MHz xtal
$hwstack = 40
$swstack = 40
$framesize = 40

'Configure LCD,  Ports C & D --------------------------------------------------
'PC.4     LCD-RS
'PC.5     LCD-E
'PD.0     LCD-DB4
'PD.1     LCD-DB5
'PD.3     LCD-DB6
'PD.4     LCD-DB7
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.4 , Db6 = PortD.3 , Db5 = PortD.1
Config Lcdpin = Pin , Db4 = PortD.0 , E = PortC.5 , Rs = PortC.4
Config Lcdbus = 4

'Configure Key port,  Ports C & D ---------------------------------------------
DDRC.0 = 0                                        'PC.0 = input
DDRC.1 = 0                                        'PC.1 = input
DDRD.6 = 0                                        'PD.6 = input
DDRD.7 = 0                                        'PD.7 = input
PortC.0 = 1                                       'Pullup PC.0
PortC.1 = 1                                       'Pullup PC.1
PortD.6 = 1                                       'Pullup PD.6
PortD.7 = 1                                       'Pullup PD.7

Dim Key_port As Byte
Const Key_sto = 0                                 'Key store @ PINC.0
Const Key_up = 1                                  'Key up    @ PIND.7
Const Key_dwn = 2                                 'Key dwn   @ PIND.6
Const Key_enc = 3                                 'Enc Key   @ PINC.1

'Configure Timer0-Interrupt for Keys (Timer0 = 8bit, 2^8=256 counts) ----------
Config Timer0 = Timer , Prescale = 1024
Const Presettimer0 = 56                           'Gives 200 timer0 counts)
'Overflow time: Overflow-Counts * Prescale / crystal frequency
'= 200 * 1024 / 16.000.000 = 12.8 msec = ~ 78 Hz (crystal 16 MHz)
'Possible Prescales: 8, 64, 256, 1024, here 1024
On Timer0 Timer0_isr                              'On Overflow go to Timer0_isr
Const bytTimer0_long = 50                         'Timer counts for long press

'Variables Keys ---------------------------------------------------------------
Dim bytIwr0 As Byte
Dim bytKey_ct0 As Byte
Dim bytKey_ct1 As Byte
Dim bytKey_state As Byte
Dim bytKey_press As Byte
Dim bytKey_save As Byte
Dim bytPressedLong As Byte
Dim bytTimer0_runs As Byte
Dim bytKey_mask As Byte
bytKey_Mask = &B00000011                          'Mask for short/long keys
'                                                  Set 1 to enable short/long

Declare Function KeyPressedLong(byVal bytKey_Long As Byte) As Byte
Declare Sub ClearLCDLine(byVal bytLine As Byte)

'Initialize Key Data
bytKey_ct0 = 0
bytKey_ct1 = 0
Decr bytKey_ct0
bytKey_ct1 = bytKey_ct0
bytKey_state = 0
bytKey_press = 0

Enable Timer0
Enable Interrupts                                 'enable all Interrupts
Timer0 = Presettimer0                             'initialize Timer0
bytTimer0_runs = 0

Cls
Cursor Off

Upperline
LCD "Taster2_104"
Lowerline
Lcd "Hello from DL6GL"
Wait 1
Cls

Do                                                'Start of main program ******

   'Check Key Key_sto short or long -------------
   If bytKey_press.Key_sto = 1 Then               'Store Key was pressed
      Call ClearLCDLine(1)
      bytPressedLong = KeyPressedLong(Key_sto)
      Select Case bytPressedLong
        Case 1                                    'Key_sto pressed short
             LCD "Key_sto short"
        Case 2                                    'Key_sto pressd long
             LCD "Key_sto long"
      End Select
   End If

   'Check Key Key_up short or long -------------
   If bytKey_press.Key_up = 1 Then                'Key_up was pressed
      Call ClearLCDLine(1)
      bytPressedLong = KeyPressedLong(Key_up)
      Select Case bytPressedLong
        Case 1                                    'Key_sto pressed short
             LCD "Key_up short"
        Case 2                                    'Key_sto pressd long
             LCD "Key_up long"
      End Select
   End If

   'Check Key Key_dwn -------------
   If bytKey_state.Key_dwn = 1 Then               'Key_dwn is still pressed
      Call ClearLCDLine(1)
      LCD "Key_dwn"
   End If

   'Check Key Key_enc -------------
   If bytKey_state.Key_enc = 1 Then               'Key_enc is still pressed
      Call ClearLCDLine(1)
      LCD "Key_enc"
   End If

Loop                                              'End of main program *******

End

'==============================================================================
Timer0_isr:
'Interrupt Service Routine for Timer0 (Keys)
'got from Thomas Baer in microcontroller.net
'Original (ASM and C) by Peter Dannegger
'http://www.mikrocontroller.net/topic/tasten-entprellen-bulletproof
'Input:  Key_port       port connected to the keys
'        bytKey_mask    Bit pattern to mask the keys enabled for short/long
'                       Key number (0...7) to be checked for short/long
'                       e.g. "00000101" Keys 0 and 2 are short/long keys
'Output: bytKey_press   bit pattern, key which changed from open to pressed
'                       Reset after evaluation in the main program
'                       Bits remain = 1 until resetted in the main program
'        bytKey_state   bit pattern, Key state, 1:pressed or 0:not pressed
'                       Bits = 1 while key is pressed, = 0: Key is released
'        bytKey_save    last state of bytKey_state before entering Timer0_isr
'                       Bits 0...7: keys 0...7 @ Key_port

   Timer0 = Presettimer0

   'Keys at different ports
   Key_port.Key_sto = PINC.0                      'Key store   @ PINC.0
   Key_port.Key_up = PIND.7                       'Key up      @ PIND.7
   Key_port.Key_dwn = PIND.6                      'Key down    @ PIND.6
   Key_port.Key_enc = PINC.1                      'Key encoder @ PINC.1

   bytKey_save = bytKey_state                     'save old status before keys were pressed
   bytIwr0 = Key_port                             'get Bits of the KeyPort
   bytIwr0 = Not bytIwr0
   bytIwr0 = bytIwr0 Xor bytKey_state
   bytKey_ct0 = bytKey_ct0 And bytIwr0
   bytKey_ct1 = bytKey_ct1 And bytIwr0
   bytKey_ct0 = Not bytKey_ct0
   bytKey_ct1 = bytKey_ct1 Xor bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct1
   bytKey_state = bytKey_state Xor bytIwr0
   bytIwr0 = bytIwr0 And bytKey_state
   bytKey_press = bytKey_press Or bytIwr0

   bytIwr0 = bytKey_state AND bytKey_mask         'Mask out short/long keys
   If bytIwr0 > 0 Then                            'Short/long key still pressed
      Incr bytTimer0_runs                         'Clock is running...
   End If
Return

'==============================================================================
Function KeyPressedLong(byVal bytKey_Long As Byte) As Byte
   'Check if key bytKey_Long (0...7) in Key_port is pressed short or long
   'Output: KeyPressedLong =  0: evaluating, key is still pressed
   '                          1: key is released, pressed short
   '                          2: key is released, pressed long

   If bytKey_state.bytKey_Long = 1 Then           'Key still pressed
      KeyPressedLong = 0
   Else                                           'Key released
      If bytTimer0_runs < bytTimer0_long Then     'short time elapsed
         KeyPressedLong = 1                       'Pressed short
      Else
         KeyPressedLong = 2                       'Pressed short
      End If
      bytTimer0_runs = 0
      bytKey_press.bytKey_Long = 0                'Reset pressed flag
   End If
End Function

'==============================================================================
Sub ClearLCDLine(byVal bytLine As Byte)
'Clear LCD line bytLine
   Locate bytline , 1
   LCD SPC(16)
   Locate bytline , 1
End Sub